import torch
import time
import sys
import tensorrt as trt
import numpy as np
import pycuda.driver as cuda
from mobilenetv2 import MobileNetV2

def saveONNX(model, filepath):
    model1 = MobileNetV2(22, 0.5)
    checkpoint = torch.load(model)
    model1.load_state_dict(checkpoint)
    model1.cuda()

    dummy_input = torch.randn(1, 3, 64, 64, device='cuda') 
    input_names = ['input']
    output_names = ['output']
    torch.onnx.export(model1, (dummy_input), filepath, verbose=True, input_names=input_names, output_names=output_names, export_params=True)

def get_engine_fp32(onnx_file_path, engine_file_path):
    G_LOGGER = trt.Logger(trt.Logger.WARNING)

    with trt.Builder(G_LOGGER) as builder, builder.create_network() as network, trt.OnnxParser(network, G_LOGGER) as parser:
        builder.max_batch_size = 1
        builder.max_workspace_size = 1 << 20  #1024

        print('Loading ONNX file from path {}...'.format(onnx_file_path))
        with open(onnx_file_path, 'rb') as model:
            print('Beginning ONNX file parsing')
            parser.parse(model.read())

        #last_layer = network.get_layer(network.num_layers - 1)
        #network.mark_output(last_layer.get_output(0))

        print('Completed parsing of ONNX file')

        print(network.num_layers)
        for i in range(network.num_layers):
            print(network.get_layer(i).name)

        print('Building an engine from file {}; this may take a while...'.format(onnx_file_path))
        engine = builder.build_cuda_engine(network)
        print("Completed creating Engine")

        with open(engine_file_path, "wb") as f:
            f.write(engine.serialize())
        return engine

if __name__ == '__main__':
    saveONNX('mbv2.pth', "mbv2.onnx")
    get_engine_fp32('mbv2.onnx',
                 'mbv2.trt')



